package com.ruoyi.system.mapper;

import java.util.List;

import com.ruoyi.common.annotation.DataSource;
import com.ruoyi.common.enums.DataSourceType;
import com.ruoyi.system.domain.TFacility;

/**
 * 手环设备Mapper接口
 *
 * @author zhengjie
 * @date 2023-08-12
 */
public interface TFacilityMapper {
    /**
     * 查询手环设备
     *
     * @param id 手环设备主键
     * @return 手环设备
     */
    public TFacility selectTFacilityById(Long id);

    /**
     * 查询设备列列表
     *
     * @return 设备列集合
     */
    public List<TFacility> selectTDeviceListListByDeviceId(String device_id_code);

    /* 查询手环设备
     *
     * @param code 手环设备
     * @return 手环设备
     */
    public TFacility selectTFacilityByCode(String code);

    /**
     * 查询手环设备列表
     *
     * @param tFacility 手环设备
     * @return 手环设备集合
     */
    public List<TFacility> selectTFacilityList(TFacility tFacility);

    /**
     * 新增手环设备
     *
     * @param tFacility 手环设备
     * @return 结果
     */
    public int insertTFacility(TFacility tFacility);

    /**
     * 修改手环设备
     *
     * @param tFacility 手环设备
     * @return 结果
     */
    public int updateTFacility(TFacility tFacility);

    /**
     * 删除手环设备
     *
     * @param id 手环设备主键
     * @return 结果
     */
    public int deleteTFacilityById(Long id);

    /**
     * 批量删除手环设备
     *
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteTFacilityByIds(Long[] ids);

    public int deleteTFacilityByCode(String deviceCode);
}
